; TWI I2C MASTER				
; 10/2005						
; Bernhard.Erfurt@gmx.de		
;								
; TAKT=1.0 MHZ					
;								
.include "m8def.inc"


; MASTER einstellungen 
.equ TWI_BIT_RATE			= 255	; Bit-Rate	(11...255				
.equ TWI_PRESCALER			= 3		; Vorteiler: 0=1 1=4 2=16 3=64		

; REGISTER				
.def ADRESSE_SLAVE	= R11

.def DATA_EMPFANG_1	= R12
.def DATA_EMPFANG_2	= R13
.def DATA_SENDEN_1	= R14
.def DATA_SENDEN_2	= R15

.def temp  			= R16
.def temp1 			= R17
.def temp2 			= R18
.def temp3 			= R19



.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0				

rjmp RESET      ; Reset Handler                           
reti ;rjmp EXT_INT0   ; IRQ0 Handler                      
reti ;rjmp EXT_INT1   ; IRQ1 Handler                      
reti ;rjmp TIM2_COMP  ; Timer2 Compare Handler            
reti ;rjmp TIM2_OVF   ; Timer2 Overflow Handler           
reti ;rjmp TIM1_CAPT  ; Timer1 Capture Handler            
reti ;rjmp TIM1_COMPA ; Timer1 CompareA Handler           
reti ;rjmp TIM1_COMPB ; Timer1 CompareB Handler           
reti ;rjmp TIM1_OVF   ; Timer1 Overflow Handler           
reti ;rjmp TIM0_OVF   ; Timer0 Overflow Handler           
reti ;rjmp SPI_STC    ; SPI Transfer Complete Handler     
reti ;rjmp USART_RXC  ; USART RX Complete Handler         
reti ;rjmp USART_UDRE ; UDR Empty Handler                 
reti ;rjmp USART_TXC  ; USART TX Complete Handler         
reti ;rjmp ADC        ; ADC Conversion Complete Handler   
reti ;rjmp EE_RDY     ; EEPROM Ready Handler              
reti ;rjmp ANA_COMP   ; Analog Comparator Handler         
reti ;rjmp TWSI       ; Two-wire Serial Interface Handler 
reti ;rjmp SPM_RDY    ; Store Program Memory Ready Handler





RESET:                        
	
;STACK initialisieren
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; PORT B + D alles AUSGNGE	
	ldi temp, 0b11111111
	out DDRB, temp
	out PORTB,temp
	out DDRD, temp
	out PORTD,temp
; PORT C alles Eingnge
	clr temp
	out DDRC,temp
; TWI-INITILISIERUNG
	rcall TWI_INI

; Startwerte		
	
	ldi temp,127			; SLAVE ADRESSE	0...127 (0=General CALL)
	mov ADRESSE_SLAVE,temp
	
	
	ldi temp, 1
	mov DATA_SENDEN_1,temp
	ldi temp, 2
	mov DATA_SENDEN_2,temp

	clr DATA_EMPFANG_1
	clr DATA_EMPFANG_2


	rcall wait_1s

LOOP:
	
	rcall TWI_MASTER_TRANSMITTER
	rcall wait_1s
	rcall wait_1s
	rcall wait_1s

	rcall TWI_MASTER_RECEIVER
	rcall wait_1s
	rcall wait_1s
	
rjmp LOOP


; #############################################################################
; #############################################################################
; #############################################################################
TWI_MASTER_RECEIVER:
; LEDs aus											
	ldi temp,255
	out PORTB,temp
; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	cbi PORTB,0
; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 	; START erfolgreich
	brne TWI_MASTER_RECEIVER_ERROR
; Load SLA +"R" (R=1)								
	mov temp, ADRESSE_SLAVE
	lsl temp		; 1 x links
	subi temp,-1
	out TWDR, temp
; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	cbi PORTB,1
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	cbi PORTB,2
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	brne TWI_MASTER_RECEIVER_ERROR
	cbi PORTB,3
; senden  "ACK" (TWEA=1) fr mehrere Daten-Bytes 
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
; TWINT abfragen									
	rcall TWI_TWINT_wait
	cbi PORTB,4
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_ERROR	; ERROR

; DATEN einlesen									
	cbi PORTB,5
	in DATA_EMPFANG_1, TWDR
; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte 
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
; TWINT abfragen									
	rcall TWI_TWINT_wait
	cbi PORTB,4
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_ERROR	; ERROR
; DATEN einlesen									
	cbi PORTB,6
	in DATA_EMPFANG_2, TWDR

; STOP												
	cbi PORTB,7
	ldi temp, (1<<TWINT | 1<<TWEN |1<<TWSTO)
	out TWCR, temp

ret
; ERROR
TWI_MASTER_RECEIVER_ERROR:
	rcall TWI_ERROR
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_MASTER_TRANSMITTER:
; alle LEDs aus										
	ldi temp,255
	out PORTB,temp
; START												
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	out TWCR, temp	
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait

; Check TWSR										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08	; 0x08
	brne TWI_MASTER_TRANSMITTER_ERROR
	cbi PORTB,0
; Load SLA + "W"  (W=0) 							
	mov temp, ADRESSE_SLAVE
	lsl temp 		; 1x links, Bit0 wird '0'
	out TWDR, temp
; senden - Clear TWINT 								
	ldi temp, (1<<TWINT) | (1<<TWEN)
	out TWCR, temp
	cbi PORTB,1
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	cbi PORTB,2
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp,  0x18	; 0x18
	brne TWI_MASTER_TRANSMITTER_ERROR
	cbi PORTB,3
; Load DATA	1										
	out TWDR, DATA_SENDEN_1
; senden - Clear TWINT 								
	ldi temp, (1<<TWINT) | (1<<TWEN)
	out TWCR, temp
	cbi PORTB,4
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	cbi PORTB,5
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MASTER_TRANSMITTER_ERROR
	cbi PORTB,6
; Load DATA	2										
	out TWDR, DATA_SENDEN_2
; senden - Clear TWINT 								
	ldi temp, (1<<TWINT) | (1<<TWEN)
	out TWCR, temp
	cbi PORTB,4
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	cbi PORTB,5
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MASTER_TRANSMITTER_ERROR
	cbi PORTB,6
; STOP												
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)
	out TWCR, temp
	cbi PORTB,7
ret
; ERROR												
TWI_MASTER_TRANSMITTER_ERROR:
	rcall TWI_ERROR

ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_INI:
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 0<<TWINT|1<<TWEA|1<<TWEN
	out TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_ERROR:
	cbi PORTD,0
; TWI STOP				
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)
	out TWCR, temp
; TWI aus				
	ldi temp, (0<<TWEN)
	out TWCR, temp
; TWI INITIALISIERUNG	
 	rcall TWI_INI
	
	rcall wait_1s
	
	sbi PORTD,0
ret
; #############################################################################
; #############################################################################
; #############################################################################
; WARTEN, bis TWINT Flag gesetzt ist 
TWI_TWINT_wait:
	in temp, TWCR
	sbrs temp, TWINT
	rjmp TWI_TWINT_wait
ret
; #############################################################################
; #############################################################################
; #############################################################################
; Warteschleife, ca. 1 Sekunde		
WAIT_1s:
	push temp1           ; temp1 auf dem Stack sichern            
	push temp2           ; temp2 auf dem Stack sichern            
	push temp3           ; temp3 auf dem Stack sichern            

;------------
	ldi	temp3,5			
	clr temp2
	clr	temp1			
wait1s_w:	
	dec	temp1
	brne	wait1s_w
	dec	temp2		
	brne	wait1s_w
	dec	temp3
	brne	wait1s_w
;------------
	pop temp3      		; temp3 wiederherstellen                    
	pop temp2      		; temp2 wiederherstellen                    
	pop temp1      		; temp1 wiederherstellen                    
ret